package sun.util.locale;

import java.util.SortedMap;
import java.util.Iterator;
import java.util.TreeSet;
import java.util.TreeMap;
import java.util.Set;
import java.util.Collections;
import java.util.Map;

public class LocaleExtensions
{
    private final Map<Character, Extension> extensionMap;
    private final String id;
    public static final LocaleExtensions CALENDAR_JAPANESE;
    public static final LocaleExtensions NUMBER_THAI;
    
    private LocaleExtensions(String id, Character key, Extension value) {
        this.id = id;
        this.extensionMap = Collections.singletonMap(key, value);
    }
    
    LocaleExtensions(Map<InternalLocaleBuilder.CaseInsensitiveChar, String> map, Set<InternalLocaleBuilder.CaseInsensitiveString> set, Map<InternalLocaleBuilder.CaseInsensitiveString, String> map2) {
        int n = !LocaleUtils.isEmpty(map);
        int n2 = !LocaleUtils.isEmpty(set);
        int n3 = !LocaleUtils.isEmpty(map2);
        if (n == 0 && n2 == 0 && n3 == 0) {
            this.id = "";
            this.extensionMap = Collections.emptyMap();
            return;
        }
        TreeMap<Character, Object> extensionMap = new TreeMap();
        if (!(n == 0)) {
            for (Map.Entry entry : map.entrySet()) {
                int lower = LocaleUtils.toLower(((InternalLocaleBuilder.CaseInsensitiveChar)entry.getKey()).value());
                String removePrivateuseVariant = (String)entry.getValue();
                if (LanguageTag.isPrivateusePrefixChar((char)lower)) {
                    removePrivateuseVariant = InternalLocaleBuilder.removePrivateuseVariant(removePrivateuseVariant);
                    if (removePrivateuseVariant == null) {
                        continue;
                    }
                }
                extensionMap.put(Character.valueOf((char)lower), new Extension((char)lower, LocaleUtils.toLowerString(removePrivateuseVariant)));
            }
        }
        if (n2 || n3) {
            Object o = null;
            Object o2 = null;
            if (!(n2 == 0)) {
                o = new TreeSet();
                Iterator<Object> iterator2 = set.iterator();
                while (iterator2.hasNext()) {
                    o.add(LocaleUtils.toLowerString(((InternalLocaleBuilder.CaseInsensitiveString)iterator2.next()).value()));
                }
            }
            if (!(n3 == 0)) {
                o2 = new TreeMap();
                for (Map.Entry entry2 : map2.entrySet()) {
                    o2.put(LocaleUtils.toLowerString(((InternalLocaleBuilder.CaseInsensitiveString)entry2.getKey()).value()), LocaleUtils.toLowerString((String)entry2.getValue()));
                }
            }
            extensionMap.put(Character.valueOf('u'), new UnicodeLocaleExtension(o, o2));
        }
        if (extensionMap.isEmpty()) {
            this.id = "";
            this.extensionMap = Collections.emptyMap();
        }
        else {
            this.id = toID(extensionMap);
            this.extensionMap = extensionMap;
        }
        return;
    }
    
    public Set<Character> getKeys() {
        if (this.extensionMap.isEmpty()) {
            return Collections.emptySet();
        }
        return Collections.unmodifiableSet(this.extensionMap.keySet());
    }
    
    public Extension getExtension(Character c) {
        return (Extension)this.extensionMap.get(Character.valueOf(LocaleUtils.toLower((char)c)));
    }
    
    public String getExtensionValue(Character c) {
        Extension extension = (Extension)this.extensionMap.get(Character.valueOf(LocaleUtils.toLower((char)c)));
        if (extension == null) {
            return null;
        }
        return extension.getValue();
    }
    
    public Set<String> getUnicodeLocaleAttributes() {
        Extension extension = (Extension)this.extensionMap.get(Character.valueOf('u'));
        if (extension == null) {
            return Collections.emptySet();
        }
        assert !(!(extension instanceof UnicodeLocaleExtension));
        return ((UnicodeLocaleExtension)extension).getUnicodeLocaleAttributes();
    }
    
    public Set<String> getUnicodeLocaleKeys() {
        Extension extension = (Extension)this.extensionMap.get(Character.valueOf('u'));
        if (extension == null) {
            return Collections.emptySet();
        }
        assert !(!(extension instanceof UnicodeLocaleExtension));
        return ((UnicodeLocaleExtension)extension).getUnicodeLocaleKeys();
    }
    
    public String getUnicodeLocaleType(String s) {
        Extension extension = (Extension)this.extensionMap.get(Character.valueOf('u'));
        if (extension == null) {
            return null;
        }
        assert !(!(extension instanceof UnicodeLocaleExtension));
        return ((UnicodeLocaleExtension)extension).getUnicodeLocaleType(LocaleUtils.toLowerString(s));
    }
    
    public boolean isEmpty() {
        return this.extensionMap.isEmpty();
    }
    
    public static boolean isValidKey(char c) {
        return LanguageTag.isExtensionSingletonChar(c) || LanguageTag.isPrivateusePrefixChar(c);
    }
    
    public static boolean isValidUnicodeLocaleKey(String s) {
        return UnicodeLocaleExtension.isKey(s);
    }
    
    private static String toID(SortedMap<Character, Extension> sortedMap) {
        StringBuilder sb = new StringBuilder();
        Extension obj = null;
        for (Map.Entry entry : sortedMap.entrySet()) {
            int charValue = (char)entry.getKey();
            Extension obj2 = (Extension)entry.getValue();
            if (LanguageTag.isPrivateusePrefixChar((char)charValue)) {
                obj = obj2;
            }
            else {
                if (sb.length() > 0) {
                    sb.append("-");
                }
                sb.append(obj2);
            }
        }
        if (obj != null) {
            if (sb.length() > 0) {
                sb.append("-");
            }
            sb.append(obj);
        }
        return sb.toString();
    }
    
    @Override
    public String toString() {
        return this.id;
    }
    
    public String getID() {
        return this.id;
    }
    
    @Override
    public int hashCode() {
        return this.id.hashCode();
    }
    
    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof LocaleExtensions && this.id.equals(((LocaleExtensions)o).id));
    }
    
    static {
        CALENDAR_JAPANESE = new LocaleExtensions("u-ca-japanese", Character.valueOf('u'), UnicodeLocaleExtension.CA_JAPANESE);
        NUMBER_THAI = new LocaleExtensions("u-nu-thai", Character.valueOf('u'), UnicodeLocaleExtension.NU_THAI);
    }
}
